def get_details():
    return dict(
        name='Http Request',
        description='Make an HTTP Request',
        is_expandable=False,
        schema=dict(
            type='object',
            additionalProperties=False,
            description='Make an HTTP Request',
            required=['inputs', 'outputs'],
            properties=dict(
                expand_results=dict(
                    type='boolean',
                    default=False,
                    enum=[False]
                ),
                inputs=dict(
                    type='object',
                    additionalProperties=False,
                    required=['project_id', 'dataset_id', 'table_id', 'columns'],
                    properties=dict(
                        method=dict(
                            type='string',
                        ),
                        url=dict(
                            type='object',
                            additionalProperties=False,
                            properties=dict(
                                oneOf=[
                                    dict(
                                        path=dict(
                                            type='string',
                                            minLength=1,
                                        ),
                                        default=dict(
                                            type='array',
                                            items=dict(
                                                type='string',
                                            )
                                        )
                                    ),
                                    dict(
                                        value=dict(
                                            type='string',
                                            format='uri'
                                        )
                                    )
                                ]
                            ),
                        ),
                        url_params=dict(
                            type='array',
                            items=dict(
                                type='object',
                                additionalProperties=False,
                                properties=dict(
                                    key=dict(
                                        type='string',
                                        minLength=1,
                                    ),
                                    value=dict(
                                        type='string',
                                    )
                                )
                            )
                        ),
                        json_payload=dict(
                            type='array',
                            items=dict(
                                type='object',
                                additionalProperties=False,
                                properties=dict(
                                    key=dict(
                                        type='string',
                                        minLength=1,
                                    ),
                                    value=dict(
                                        oneOf=[
                                            dict(
                                                path=dict(
                                                    type='string',
                                                    minLength=1,
                                                ),
                                                default=dict(
                                                    type='string',
                                                )
                                            ),
                                            dict(
                                                value=dict(
                                                    oneOf=[
                                                        dict(
                                                            type='string',
                                                        ),
                                                        dict(
                                                            type='number',
                                                        ),
                                                        dict(
                                                            type='boolean',
                                                        ),
                                                        dict(
                                                            type='array',
                                                            items=dict(
                                                                type='string',
                                                            )
                                                        ),
                                                        dict(
                                                            type='object',
                                                        )
                                                    ]
                                                )
                                            )
                                        ]
                                    )
                                )
                            )
                        ),
                    )
                ),
                outputs=dict(
                    type='object',
                    required=[
                        'status_code'
                    ],
                    properties=dict(
                        status_code=dict(
                            type='integer',
                        ),
                        json=dict(
                            type='object',
                        ),
                        text=dict(
                            type='string',
                        ),
                    )
                ),
            )
        )
    )


def process(inputs,
            **kwargs):
    import requests

    resp = requests.request(
        method=inputs['method'],
        url=inputs['url'],
        params=dict(
            (param['key'], param['value']) for param in inputs['url_params']
        ),
        json=dict(
            (param['key'], param['value']) for param in inputs['json_payload']
        )
    )
    response = dict(
        status_code=resp.status_code,
        text=resp.text
    )
    try:
        response['json'] = resp.json()
    except:
        pass
    return response



def get_details():
    return dict(
        name='Create Account',
        description='Create account based on provided data',
        is_expandable=False,
        schema=dict(
            type='object',
            additionalProperties=False,
            description='Insert Row into BigQuery Table',
            required=['inputs', 'outputs'],
            properties=dict(
                expand_results=dict(
                    type='boolean',
                    default=False,
                    enum=[False]
                ),
                inputs=dict(
                    type='object',
                    additionalProperties=False,
                    required=['project_id', 'dataset_id', 'table_id', 'columns'],
                    properties=dict(
                        parent_uuids=dict(
                            type='object',
                            additionalProperties=False,
                            properties=dict(
                                oneOf=[
                                    dict(
                                        path=dict(
                                            type='string',
                                            minLength=1,
                                        ),
                                        default=dict(
                                            type='array',
                                            items=dict(
                                                type='string',
                                                format='uuid'
                                            )
                                        )
                                    ),
                                    dict(
                                        value=dict(
                                            type='array',
                                            items=dict(
                                                type='string',
                                                format='uuid'
                                            )
                                        )
                                    )
                                ]
                            ),
                        ),
                        name=dict(
                            type='object',
                            additionalProperties=False,
                            properties=dict(
                                oneOf=[
                                    dict(
                                        path=dict(
                                            type='string',
                                            minLength=1,
                                        ),
                                        default=dict(
                                            type='string',
                                        )
                                    ),
                                    dict(
                                        value=dict(
                                            type='string',
                                        )
                                    )
                                ]
                            ),
                        ),
                        account_type_uuid=dict(
                            type='object',
                            additionalProperties=False,
                            properties=dict(
                                oneOf=[
                                    dict(
                                        path=dict(
                                            type='string',
                                            minLength=1,
                                        ),
                                        default=dict(
                                            type='string',
                                            format='uuid'
                                        )
                                    ),
                                    dict(
                                        value=dict(
                                            type="string",
                                            format='uuid'
                                        ),

                                    )
                                ]
                            ),
                        ),
                        extra_attributes=dict(
                            type='object',
                            additionalProperties=False,
                            properties=dict(
                                oneOf=[
                                    dict(
                                        path=dict(
                                            type='string',
                                            minLength=1,
                                        ),
                                        default=dict(
                                            type='object',
                                        )
                                    ),
                                    dict(
                                        value=dict(
                                            type='object',
                                        )
                                    )
                                ]
                            )
                        ),
                    )
                ),
                outputs=dict(
                    type='object',
                    required=[
                        'account_uuid'
                        'parent_uuids',
                        'name',
                        'created_ts'
                    ],
                    properties=dict(
                        account_uuid=dict(
                            type='string',
                        ),
                        parent_uuids=dict(
                            type='array',
                            items=dict(
                                type='string',
                                format='uuid'
                            )
                        ),
                        name=dict(
                            type='string'
                        ),
                        created_ts=dict(
                            type='integer',
                        )
                    )
                ),
            )
        )
    )


def process(workflow_uuid, account_uuid, node_uuid, trigger_uuid, ingested_ts, processed_ts, inputs, integration,
            **kwargs):
    pass



from importlib import import_module


def load_integration_type(integration_key):
    module = import_module(
        name=integration_key
    )
    return module


def load_operation(integration_key, operation_key):
    module = import_module(
        name=f'{integration_key}.{operation_key}'
    )
    return module


def format_data(record, schema):
    from jsonpath_ng.ext.parser import parse as ng_parse
    data = dict()
    for k, v in schema.items():
        val = None
        if isinstance(v, list):
            val = [format_data(record=record, schema=i) for i in v]
        elif isinstance(v, dict):
            if all(i in ['value', 'path', 'default'] for i in v.keys()):
                if 'value' in v.keys():
                    val = v['value']
                elif 'path' in v.keys():
                    parser = ng_parse(v['path'])
                    val = [i.value for i in parser.find(record)]
                    if len(val) == 0:
                        val = None
                    elif len(val) == 1:
                        val = val[0]
                if val is None and 'default' in v.keys():
                    val = v['default']
            else:
                val = v
        data[k] = val
    return data
